import json
import os, os.path
import pathlib
import sys
import tempfile

from deppth import deppth # not sure why the nested path
from PIL import Image, ImageDraw


def get_list(filename):
    d = pathlib.Path(__file__).parent
    lines = (d / filename).read_text().splitlines()
    return set(lines)


def remove_subtexture(package_dir, manifest, sub_atlas):
    filename = manifest['name'].split(os.path.sep)[-1] + '.png'
    file = package_dir / 'textures/atlases' / filename
    img = Image.open(file)

    draw = ImageDraw.Draw(img)
    r = sub_atlas['rect']
    top_left = (r['x'], r['y'])
    bottom_right = (r['x'] + r['width'], r['y'] + r['height'])
    draw.rectangle(xy=(top_left, bottom_right), fill=(0, 0, 0, 0))

    img.save(file)


def process_pkg(src_path, include, exclude):
    print('Processing package', src_path,)
    with tempfile.TemporaryDirectory() as tmp:
        tmp = pathlib.Path(tmp)
        pkg_dir = str(tmp / 'package')
        print('Extracting package - this may take some time.')
        deppth.extract(src_path, pkg_dir)

        manifests = []
        for m in (tmp / 'package/manifest').iterdir():
            with open(m) as f:
                manifests.append(json.load(f))

        print('Removing subtextures')
        modified = []
        for manifest in manifests:
            for sub in manifest['subAtlases']:
                if 'aphrodite' in sub['name'].lower():
                    if sub['name'] in include:
                        remove_subtexture(tmp / 'package', manifest, sub)
                        manifest_name = manifest['name'].split(os.path.sep)[-1]
                        modified.append(manifest_name)
                    elif sub['name'] not in exclude:
                        print(f'WARNING: Unknown texture "{sub["name"]}". You may wish to add this texture to include.txt or exclude.txt.')

        print('Constructing patches')
        patch_path = str(tmp / 'patch.pkg')
        deppth.pack(pkg_dir, patch_path, *modified)
        print('Repacking package')
        deppth.patch(src_path, patch_path)
        print('Finished processing package.')


def run():
    if len(sys.argv) > 1:
        steam_dir = sys.argv[1]
    else:
        steam_dir = 'C:/Program Files (x86)/Steam'
    base_dir = pathlib.Path(steam_dir) / 'steamapps/common/Hades/Content/Win/Packages'
    if not base_dir.exists():
        print(f'Could not find Hades game files. Is {steam_dir} the correct location for your Steam library?')
        return 1

    include = get_list('include.txt')
    exclude = get_list('exclude.txt')
    for pkg in 'GUI.pkg', '720p/GUI.pkg', 'BC3/GUI.pkg':
        path = str(base_dir / pkg)
        process_pkg(path, include, exclude)
        print()
    print('Finished.')


if __name__ == '__main__':
    run()
